@@ -1,4 +1,5 @@
 #!/usr/bin/env py.test
+# -*- coding: utf-8 -*-
 """
 This file contains snippets from the FEniCS book,
 and allows us to test that these can still run
@@ -159,7 +160,8 @@ def test_uflcode_824(self):
 def test_uflcode_886(self):
     cell = triangle
     # ...
-    g = sin(cell.x[0])
+    x = SpatialCoordinate(cell) # Original: x = cell.x
+    g = sin(x[0])
     v = variable(g)
     f = exp(v**2)
     h = diff(f, v)
@@ -342,7 +344,7 @@ def test_python_1843(self):
         if e._ufl_is_terminal_:
             return e
         ops = [apply_ad(o, ad_routine) for o in e.ufl_operands]
-        e = e.reconstruct(*ops)
+        e = e._ufl_expr_reconstruct_(*ops)
         if isinstance(e, Derivative):
             e = ad_routine(e)
         return e
@@ -417,7 +419,7 @@ def test_python_2024(self):
 
     cell = triangle
     element = FiniteElement("Lagrange", cell, 1)
-    x = cell.x
+    x = SpatialCoordinate(cell) # Original: x = cell.x
     if 0:
         print((m(Argument(element, 2))))
         print((m(x)))
@@ -436,7 +438,7 @@ def test_python_2087(self):
             self.mapping = mapping
 
         def operator(self, e, *ops):
-            return e.reconstruct(*ops)
+            return e._ufl_expr_reconstruct_(*ops)
 
         def terminal(self, e):
             return self.mapping.get(e, e)
@@ -471,7 +473,7 @@ def test_python_2189(self):
 
 def test_python_2328(self):
     cell = triangle
-    x = cell.x
+    x = SpatialCoordinate(cell) # Original: x = cell.x
     e = x[0] + x[1]
     #print e((0.5, 0.7)) # prints 1.2
     # ...
@@ -479,7 +481,7 @@ def test_python_2328(self):
 
 def test_python_2338(self):
     cell = triangle
-    x = cell.x
+    x = SpatialCoordinate(cell) # Original: x = cell.x
     # ...
     c = Constant(cell)
     e = c*(x[0] + x[1])
