@@ -1,12 +1,14 @@
-
+#!/usr/bin/env py.test
+# -*- coding: utf-8 -*-
 import pytest
 from ufl import *
 from ufl.algorithms.compute_form_data import compute_form_data
 
 def test_check_arities():
     # Code from bitbucket issue #49
-    D = Domain(tetrahedron)
-    V = VectorElement("P", D, 2)
+    cell = tetrahedron
+    D = Mesh(cell)
+    V = FunctionSpace(D, VectorElement("P", cell, 2))
     dv = TestFunction(V)
     du = TrialFunction(V)
 
