@@ -1,6 +1,7 @@
 #!/usr/bin/env py.test
+# -*- coding: utf-8 -*-
 
-__authors__ = "Martin Sandve Alnes"
+__authors__ = "Martin Sandve Alnæs"
 __date__ = "2009-02-17 -- 2009-02-17"
 
 import pytest
@@ -39,8 +40,8 @@ def assertEqualBySampling(actual, expected):
     amapping = dict((c, make_value(c)) for c in chain(ad.original_form.coefficients(), ad.original_form.arguments()))
     bmapping = dict((c, make_value(c)) for c in chain(bd.original_form.coefficients(), bd.original_form.arguments()))
 
-    acell = actual.cell()
-    bcell = expected.cell()
+    acell = actual.ufl_domain().ufl_cell()
+    bcell = expected.ufl_domain().ufl_cell()
     assert acell == bcell
     if acell.geometric_dimension() == 1:
         x = (0.3,)
@@ -390,11 +391,11 @@ def test_coefficient_derivatives(self):
 
     dv = TestFunction(V)
 
-    f = Coefficient(V).reconstruct(count=0)
-    g = Coefficient(V).reconstruct(count=1)
-    df = Coefficient(V).reconstruct(count=2)
-    dg = Coefficient(V).reconstruct(count=3)
-    u = Coefficient(V).reconstruct(count=4)
+    f = Coefficient(V, count=0)
+    g = Coefficient(V, count=1)
+    df = Coefficient(V, count=2)
+    dg = Coefficient(V, count=3)
+    u = Coefficient(V, count=4)
     cd = { f: df, g: dg }
 
     integrand = inner(f, g)
@@ -413,10 +414,10 @@ def test_vector_coefficient_derivatives(self):
 
     dv = TestFunction(V)
 
-    df = Coefficient(VV).reconstruct(count=0)
-    g = Coefficient(V).reconstruct(count=1)
-    f = Coefficient(V).reconstruct(count=2)
-    u = Coefficient(V).reconstruct(count=3)
+    df = Coefficient(VV, count=0)
+    g = Coefficient(V, count=1)
+    f = Coefficient(V, count=2)
+    u = Coefficient(V, count=3)
     cd = { f: df }
 
     integrand = inner(f, g)
@@ -437,11 +438,11 @@ def test_vector_coefficient_derivatives_of_product(self):
 
     dv = TestFunction(V)
 
-    df = Coefficient(VV).reconstruct(count=0)
-    g = Coefficient(V).reconstruct(count=1)
-    dg = Coefficient(VV).reconstruct(count=2)
-    f = Coefficient(V).reconstruct(count=3)
-    u = Coefficient(V).reconstruct(count=4)
+    df = Coefficient(VV, count=0)
+    g = Coefficient(V, count=1)
+    dg = Coefficient(VV, count=2)
+    f = Coefficient(V, count=3)
+    u = Coefficient(V, count=4)
     cd = { f: df, g: dg }
 
     integrand = f[i]*g[i]
