@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """The Unified Form Language is an embedded domain specific language
 for definition of variational forms intended for finite element
 discretization. More precisely, it defines a fixed interface for choosing
@@ -38,15 +39,14 @@ The development version can be found in the repository at
 
 A very brief overview of the language contents follows:
 
-* Domains::
-
-    Domain, ProductDomain
-
 * Cells::
 
-    Cell, ProductCell, OuterProductCell,
-    interval, triangle, tetrahedron,
-    quadrilateral, hexahedron
+    AbstractCell, Cell, TensorProductCell, OuterProductCell,
+    vertex, interval, triangle, tetrahedron, quadrilateral, hexahedron
+
+* Domains::
+
+    AbstractDomain, Mesh, MeshView, TensorProductMesh
 
 * Sobolev spaces::
 
@@ -58,13 +58,18 @@ A very brief overview of the language contents follows:
     MixedElement, VectorElement, TensorElement
     EnrichedElement, RestrictedElement,
     TensorProductElement, OuterProductElement,
-    OuterProductVectorElement, HDiv, HCurl
+    HDivElement, HCurlElement
     BrokenElement, TraceElement
     FacetElement, InteriorElement
 
+* Function spaces::
+
+    FunctionSpace
+
 * Arguments::
 
-    Argument, TestFunction, TrialFunction
+    Argument, TestFunction, TrialFunction,
+    Arguments, TestFunctions, TrialFunctions
 
 * Coefficients::
 
@@ -84,6 +89,7 @@ A very brief overview of the language contents follows:
     FacetNormal, CellNormal,
     CellVolume, Circumradius, MinCellEdgeLength, MaxCellEdgeLength,
     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
+    Jacobian, JacobianDeterminant, JacobianInverse
 
 * Indices::
 
@@ -139,7 +145,7 @@ A very brief overview of the language contents follows:
 * Integral measures::
 
     dx, ds, dS, dP,
-    dc, dC, dO, dI,
+    dc, dC, dO, dI, dX
     ds_b, ds_t, ds_tb, ds_v, dS_h, dS_v
 
 * Form transformations::
@@ -150,7 +156,7 @@ A very brief overview of the language contents follows:
     sensitivity_rhs, derivative
 """
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2015 Martin Sandve Alnæs and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -173,7 +179,7 @@ A very brief overview of the language contents follows:
 # Modified by Andrew T. T. McRae, 2014
 # Modified by Lawrence Mitchell, 2014
 
-__version__ = "1.6.0"
+__version__ = "2016.1.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -184,20 +190,22 @@ __version__ = "1.6.0"
 
 # Utility functions (product is the counterpart of the built-in
 # python function sum, can be useful for users as well?)
-from ufl.common import product
+from ufl.utils.sequences import product
 
 # Output control
 from ufl.log import get_handler, get_logger, set_handler, set_level, add_logfile, \
     UFLException, DEBUG, INFO, WARNING, ERROR, CRITICAL
 
 # Types for geometric quantities
-from ufl.cell import as_cell, Cell, ProductCell, OuterProductCell
-from ufl.domain import as_domain, Domain, ProductDomain
+
+from ufl.cell import as_cell, AbstractCell, Cell, TensorProductCell, OuterProductCell
+from ufl.domain import as_domain, AbstractDomain, Mesh, MeshView, TensorProductMesh
 from ufl.geometry import (
     SpatialCoordinate,
     FacetNormal, CellNormal,
     CellVolume, Circumradius, MinCellEdgeLength, MaxCellEdgeLength,
     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
+    Jacobian, JacobianDeterminant, JacobianInverse
     )
 
 # Sobolev spaces
@@ -207,12 +215,15 @@ from ufl.sobolevspace import L2, H1, H2, HDiv, HCurl
 from ufl.finiteelement import FiniteElementBase, FiniteElement, \
     MixedElement, VectorElement, TensorElement, EnrichedElement, \
     RestrictedElement, TensorProductElement, OuterProductElement, \
-    OuterProductVectorElement, HDiv, HCurl, BrokenElement, TraceElement, \
+    HDivElement, HCurlElement, BrokenElement, TraceElement, \
     FacetElement, InteriorElement
 
 # Hook to extend predefined element families
 from ufl.finiteelement.elementlist import register_element, show_elements #, ufl_elements
 
+# Function spaces
+from ufl.functionspace import FunctionSpace, MixedFunctionSpace
+
 # Arguments
 from ufl.argument import Argument, TestFunction, TrialFunction, \
                          Arguments, TestFunctions, TrialFunctions
@@ -257,7 +268,7 @@ from ufl.operators import rank, shape, \
                        elem_mult, elem_div, elem_pow, elem_op
 
 # Measure classes
-from ufl.measure import Measure, register_integral_type, integral_types
+from ufl.measure import Measure, register_integral_type, integral_types, custom_integral_types
 
 # Form class
 from ufl.form import Form, replace_integral_domains
@@ -271,7 +282,7 @@ import ufl.measureoperators as __measureoperators
 
 # Representations of transformed forms
 from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs,\
-    system, functional, adjoint, sensitivity_rhs #, dirichlet_functional
+    system, functional, adjoint, sensitivity_rhs, block_split #, dirichlet_functional
 
 # Predefined convenience objects
 from ufl.objects import (
@@ -279,7 +290,7 @@ from ufl.objects import (
     quadrilateral, hexahedron, facet,
     i, j, k, l, p, q, r, s,
     dx, ds, dS, dP,
-    dc, dC, dO, dI,
+    dc, dC, dO, dI, dX,
     ds_b, ds_t, ds_tb, ds_v, dS_h, dS_v
     )
 
@@ -290,19 +301,21 @@ __all__ = [
     'product',
     'get_handler', 'get_logger', 'set_handler', 'set_level', 'add_logfile',
     'UFLException', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL',
-    'as_cell', 'Cell', 'ProductCell', 'OuterProductCell',
-    'as_domain', 'Domain', 'ProductDomain',
+    'as_cell', 'AbstractCell', 'Cell', 'TensorProductCell', 'OuterProductCell',
+    'as_domain', 'AbstractDomain', 'Mesh', 'MeshView', 'TensorProductMesh',
     'L2', 'H1', 'H2', 'HCurl', 'HDiv',
     'SpatialCoordinate',
     'CellVolume', 'Circumradius', 'MinCellEdgeLength', 'MaxCellEdgeLength',
     'FacetArea', 'MinFacetEdgeLength', 'MaxFacetEdgeLength',
     'FacetNormal', 'CellNormal',
+    'Jacobian', 'JacobianDeterminant', 'JacobianInverse',
     'FiniteElementBase', 'FiniteElement',
     'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement',
     'RestrictedElement', 'TensorProductElement', 'OuterProductElement',
-    'OuterProductVectorElement', 'HDiv', 'HCurl',
+    'HDivElement', 'HCurlElement',
     'BrokenElement', 'TraceElement', 'FacetElement', 'InteriorElement',
     'register_element', 'show_elements',
+    'FunctionSpace',
     'Argument', 'TestFunction', 'TrialFunction',
     'Arguments', 'TestFunctions', 'TrialFunctions',
     'Coefficient', 'Coefficients',
@@ -328,11 +341,11 @@ __all__ = [
     'jump', 'avg', 'cell_avg', 'facet_avg',
     'elem_mult', 'elem_div', 'elem_pow', 'elem_op',
     'Form',
-    'Integral', 'Measure', 'register_integral_type', 'integral_types',
-    'replace', 'replace_integral_domains', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
+    'Integral', 'Measure', 'register_integral_type', 'integral_types', 'custom_integral_types',
+    'replace', 'replace_integral_domains', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs', 'block_split',
     'system', 'functional', 'adjoint', 'sensitivity_rhs',
     'dx', 'ds', 'dS', 'dP',
-    'dc', 'dC', 'dO', 'dI',
+    'dc', 'dC', 'dO', 'dI', 'dX',
     'ds_b', 'ds_t', 'ds_tb', 'ds_v', 'dS_h', 'dS_v',
     'vertex', 'interval', 'triangle', 'tetrahedron',
     'quadrilateral', 'hexahedron', 'facet',
