@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 """This module defines partial differentiation rules for
 all relevant operands for use with reverse mode AD."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2015 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -19,9 +20,6 @@ all relevant operands for use with reverse mode AD."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard, 2011
-#
-# First added:  2009-01-06
-# Last changed: 2012-04-12
 
 from math import pi
 from ufl.log import error
@@ -35,8 +33,6 @@ class PartialDerivativeComputer(MultiFunction):
     """NB! The main reason for keeping this out of the Expr hierarchy is
     to avoid user mistakes in the form of mixups with total derivatives,
     and to allow both reverse and forward mode AD."""
-    #def __init__(self, spatial_dim):
-        #self._spatial_dim = spatial_dim
     def __init__(self):
         MultiFunction.__init__(self)
 
@@ -55,12 +51,11 @@ class PartialDerivativeComputer(MultiFunction):
 
     def sum(self, f):
         "d/dx_i sum_j x_j = 1"
-        #_1 = IntValue(1, o.free_indices(), o.index_dimensions())
         _1 = IntValue(1) # TODO: Handle non-scalars
-        return (_1,)*len(f.operands())
+        return (_1,)*len(f.ufl_operands)
 
     def product(self, f):
-        a, b = f.operands() # TODO: Assuming binary operator for now
+        a, b = f.ufl_operands # TODO: Assuming binary operator for now
         da = b # TODO: Is this right even for non-scalar b?
         db = a
         return (da, db)
@@ -69,10 +64,10 @@ class PartialDerivativeComputer(MultiFunction):
         """f = x/y
         d/dx x/y = 1/y
         d/dy x/y = -x/y**2 = -f/y"""
-        x, y = f.operands()
+        x, y = f.ufl_operands
         # Nonscalar x not supported
-        ufl_assert(x.shape() == (), "Expecting scalars in division.")
-        ufl_assert(y.shape() == (), "Expecting scalars in division.")
+        ufl_assert(x.ufl_shape == (), "Expecting scalars in division.")
+        ufl_assert(y.ufl_shape == (), "Expecting scalars in division.")
         d = 1 / y
         return (d, -f*d)
 
@@ -80,14 +75,14 @@ class PartialDerivativeComputer(MultiFunction):
         """f = x**y
         d/dx x**y = y*x**(y-1) = y*f/x
         d/dy x**y = ln(x)*x**y = ln(x)*f"""
-        x, y = f.operands()
+        x, y = f.ufl_operands
         dx = y*f/x
         dy = ln(x)*f
         return (dx, dy)
 
     def abs(self, f):
         r".. math:: \\frac{d}{dx} f(x) = \\frac{d}{dx} abs(x) = sign(x)"
-        x, = f.operands()
+        x, = f.ufl_operands
         dx = sign(x)
         return (dx,)
 
@@ -103,67 +98,67 @@ class PartialDerivativeComputer(MultiFunction):
 
     def ln(self, f):
         "d/dx ln x = 1 / x"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (1/x,)
 
     def cos(self, f):
         "d/dx cos x = -sin(x)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (-sin(x),)
 
     def sin(self, f):
         "d/dx sin x = cos(x)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (cos(x),)
 
     def tan(self, f):
         "d/dx tan x = (sec(x))^2 = 2/(cos(2x) + 1)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (2.0/(cos(2.0*x) + 1.0),)
 
     def cosh(self, f):
         "d/dx cosh x = sinh(x)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (sinh(x),)
 
     def sinh(self, f):
         "d/dx sinh x = cosh(x)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (cosh(x),)
 
     def tanh(self, f):
         "d/dx tanh x = (sech(x))^2 = (2 cosh(x) / (cosh(2x) + 1))^2"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (((2.0*cosh(x))/(cosh(2.0*x) + 1.0))**2,)
 
     def acos(self, f):
         r".. math:: \\frac{d}{dx} f(x) = \frac{d}{dx} \arccos(x) = \frac{-1}{\sqrt{1 - x^2}}"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (-1.0/sqrt(1.0 - x**2),)
 
     def asin(self, f):
         "d/dx asin x = 1/sqrt(1 - x^2)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (1.0/sqrt(1.0 - x**2),)
 
     def atan(self, f):
         "d/dx atan x = 1/(1 + x^2)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (1.0/(1.0 + x**2),)
 
     def atan_2(self, f):
         """
         f = atan2(x,y)
-        d/dx atan2(x,y) = y / (x**2 + y**2 ) 
+        d/dx atan2(x,y) = y / (x**2 + y**2 )
         d/dy atan2(x,y) = -x / (x**2 + y**2)
         """
-        x,y = f.operands()
+        x, y = f.ufl_operands
         d = x**2 + y**2
-        return (y/d,-x/d)
+        return (y/d, -x/d)
 
     def erf(self, f):
         "d/dx erf x = 2/sqrt(pi)*exp(-x^2)"
-        x, = f.operands()
+        x, = f.ufl_operands
         return (2.0/sqrt(pi)*exp(-x**2),)
 
     def bessel_function(self, nu, x):
@@ -173,22 +168,22 @@ class PartialDerivativeComputer(MultiFunction):
 
     def indexed(self, f): # TODO: Is this right? Fix for non-scalars too.
         "d/dx x_i = (1)_i = 1"
-        s = f.shape()
+        s = f.ufl_shape
         ufl_assert(s == (), "TODO: Assuming a scalar expression.")
         _1 = IntValue(1) # TODO: Non-scalars
         return (_1, None)
 
     def list_tensor(self, f): # TODO: Is this right? Fix for higher order tensors too.
         "d/dx_i [x_0, ..., x_n-1] = e_i (unit vector)"
-        ops = f.operands()
+        ops = f.ufl_operands
         n = len(ops)
-        s = ops[0].shape()
+        s = ops[0].ufl_shape
         ufl_assert(s == (), "TODO: Assuming a vector, i.e. scalar operands.")
         return unit_vectors(n) # TODO: Non-scalars
 
     def component_tensor(self, f):
-        x, i = f.operands()
-        s = f.shape()
+        x, i = f.ufl_operands
+        s = f.ufl_shape
         ufl_assert(len(s) == 1, "TODO: Assuming a vector, i.e. scalar operands.")
         n, = s
         d = ListTensor([1]*n) # TODO: Non-scalars
@@ -217,8 +212,8 @@ class PartialDerivativeComputer(MultiFunction):
         return (None, None)
 
     def conditional(self, f): # TODO: Is this right? What about non-scalars?
-        c, a, b = f.operands()
-        s = f.shape()
+        c, a, b = f.ufl_operands
+        s = f.ufl_shape
         ufl_assert(s == (), "TODO: Assuming scalar valued expressions.")
         _0 = Zero()
         _1 = IntValue(1)
@@ -229,25 +224,24 @@ class PartialDerivativeComputer(MultiFunction):
     # --- Derivatives
 
     def spatial_derivative(self, f):
-        error("Partial derivative of spatial_derivative not implemented, "\
+        error("Partial derivative of spatial_derivative not implemented, "
               "when is this called? apply_ad should make sure it isn't called.")
-        x, i = f.operands()
+        x, i = f.ufl_operands
         return (None, None)
 
     def variable_derivative(self, f):
-        error("Partial derivative of variable_derivative not implemented, "\
+        error("Partial derivative of variable_derivative not implemented, "
               "when is this called? apply_ad should make sure it isn't called.")
-        x, v = f.operands()
+        x, v = f.ufl_operands
         return (None, None)
 
     def coefficient_derivative(self, f):
-        error("Partial derivative of coefficient_derivative not implemented, "\
+        error("Partial derivative of coefficient_derivative not implemented, "
               "when is this called? apply_ad should make sure it isn't called.")
-        a, w, v = f.operands()
+        a, w, v = f.ufl_operands
         return (None, None, None)
 
 # Example usage:
 def pdiffs(exprs):
     pd = PartialDerivativeComputer()
     return [pd(e) for e in exprs]
-
