@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 """Signature computation for forms."""
 
-# Copyright (C) 2012-2014 Martin Sandve Alnes
+# Copyright (C) 2012-2015 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -24,9 +25,9 @@ from ufl.classes import (Terminal, Label,
                          GeometricQuantity, ConstantValue,
                          ExprList, ExprMapping)
 from ufl.log import error
-from ufl.corealg.traversal import traverse_unique_terminals
-from ufl.common import pre_traversal, sorted_by_count
-from ufl.geometry import join_domains
+from ufl.corealg.traversal import traverse_unique_terminals, pre_traversal
+from ufl.utils.sorting import sorted_by_count
+from ufl.domain import join_domains
 from ufl.algorithms.domain_analysis import canonicalize_metadata
 
 def compute_multiindex_hashdata(expr, index_numbering):
@@ -65,16 +66,16 @@ def compute_terminal_hashdata(expressions, renumbering):
                 data = compute_multiindex_hashdata(expr, index_numbering)
 
             elif isinstance(expr, ConstantValue):
-                data = expr.signature_data(renumbering)
+                data = expr._ufl_signature_data_(renumbering)
 
             elif isinstance(expr, Coefficient):
-                data = expr.signature_data(renumbering)
+                data = expr._ufl_signature_data_(renumbering)
 
             elif isinstance(expr, Argument):
-                data = expr.signature_data(renumbering)
+                data = expr._ufl_signature_data_(renumbering)
 
             elif isinstance(expr, GeometricQuantity):
-                data = expr.signature_data(renumbering)
+                data = expr._ufl_signature_data_(renumbering)
 
             elif isinstance(expr, Label):
                 # Numbering labels as we visit them # TODO: Include in renumbering
@@ -107,52 +108,13 @@ def compute_expression_hashdata(expression, terminal_hashdata):
         if expr._ufl_is_terminal_:
             data = terminal_hashdata[expr]
         else:
-            data = expr._ufl_typecode_ # TODO: Use expr.signature_data()? More extensible, but more overhead.
+            data = expr._ufl_typecode_ # TODO: Use expr._ufl_signature_data_()? More extensible, but more overhead.
         expression_hashdata.append(data)
     # Oneliner: TODO: Benchmark, maybe use a generator?
     #expression_hashdata = [(terminal_hashdata[expr] if expr._ufl_is_terminal_ else expr._ufl_typecode_)
     #                       for expr in pre_traversal(expression)]
     return expression_hashdata
 
-def build_domain_numbering(domains):
-    # Create canonical numbering of domains for stable signature
-    # (ordering defined by __lt__ implementation in Domain class)
-    assert None not in domains
-
-    # Collect domain keys
-    items = []
-    for i, domain in enumerate(domains):
-        key = (domain.cell(), domain.label())
-        items.append((key, i))
-
-    # Build domain numbering, not allowing repeated keys
-    domain_numbering = {}
-    for key, i in items:
-        if key in domain_numbering:
-            error("Domain key %s occured twice!" % (key,))
-        domain_numbering[key] = i
-
-    # Build domain numbering extension for None-labeled domains, not allowing ambiguity
-    from collections import defaultdict
-    domain_numbering2 = defaultdict(list)
-    for key, i in items:
-        cell, label = key
-        key2 = (cell, None)
-        domain_numbering2[key2].append(domain_numbering[key])
-
-    # Add None-based key only where unambiguous
-    for key, i in items:
-        cell, label = key
-        key2 = (cell, None)
-        if len(domain_numbering2[key2]) == 1:
-            domain_numbering[key2] = domain_numbering[key]
-        else:
-            # Two domains occur with same properties but different label,
-            # so we cannot decide which one to map None-labeled Domains to.
-            pass
-
-    return domain_numbering
-
 def compute_expression_signature(expr, renumbering): # FIXME: Fix callers
     # FIXME: Rewrite in terms of compute_form_signature?
 
@@ -182,7 +144,7 @@ def compute_form_signature(form, renumbering): # FIXME: Fix callers
         integrand_hashdata = compute_expression_hashdata(integral.integrand(),
                                                           terminal_hashdata)
 
-        domain_hashdata = integral.domain().signature_data(renumbering)
+        domain_hashdata = integral.ufl_domain()._ufl_signature_data_(renumbering)
 
         # Collect all data about integral that should be reflected in signature,
         # including compiler data but not domain data, because compiler data
