@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 """This module defines the Coefficient class and a number
 of related classes, including Constant."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2015 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -20,13 +21,15 @@ of related classes, including Constant."""
 #
 # Modified by Anders Logg, 2008-2009.
 
-from ufl.log import warning
+from ufl.log import deprecate
 from ufl.assertions import ufl_assert
+from ufl.core.ufl_type import ufl_type
 from ufl.core.terminal import Terminal, FormArgument
 from ufl.finiteelement import FiniteElementBase, FiniteElement, VectorElement, TensorElement
+from ufl.domain import as_domain, default_domain
+from ufl.functionspace import AbstractFunctionSpace, FunctionSpace
 from ufl.split_functions import split
-from ufl.common import counted_init
-from ufl.core.ufl_type import ufl_type
+from ufl.utils.counted import counted_init
 
 # --- The Coefficient class represents a coefficient in a form ---
 
@@ -35,64 +38,63 @@ class Coefficient(FormArgument):
     """UFL form argument type: Representation of a form coefficient."""
 
     # Slots are disabled here because they cause trouble in PyDOLFIN multiple inheritance pattern:
-    #__slots__ = ("_count", "_element", "_repr", "_gradient", "_derivatives")
+    #__slots__ = ("_count", "_ufl_function_space", "_repr", "_ufl_shape")
     _ufl_noslots_ = True
     _globalcount = 0
 
-    def __init__(self, element, count=None):
+    def __init__(self, function_space, count=None):
         FormArgument.__init__(self)
         counted_init(self, count, Coefficient)
 
-        ufl_assert(isinstance(element, FiniteElementBase),
-            "Expecting a FiniteElementBase instance.")
-        self._element = element
-        self._repr = None
+        if isinstance(function_space, FiniteElementBase):
+            # For legacy support for .ufl files using cells, we map the cell to The Default Mesh
+            element = function_space
+            domain = default_domain(element.cell())
+            function_space = FunctionSpace(domain, element)
+        elif not isinstance(function_space, AbstractFunctionSpace):
+            error("Expecting a FunctionSpace or FiniteElement.")
+
+        self._ufl_function_space = function_space
+        self._ufl_shape = function_space.ufl_element().value_shape()
+
+        self._repr = "Coefficient(%r, %r)" % (self._ufl_function_space, self._count)
 
     def count(self):
         return self._count
 
-    def reconstruct(self, element=None, count=None):
-        # This code is shared with the FooConstant classes
-        if element is None or element == self._element:
-            element = self._element
-        if count is None or count == self._count:
-            count = self._count
-        if count is self._count and element is self._element:
-            return self
-        ufl_assert(isinstance(element, FiniteElementBase),
-                   "Expecting an element, not %s" % element)
-        ufl_assert(isinstance(count, int),
-                   "Expecting an int, not %s" % count)
-        ufl_assert(element.value_shape() == self._element.value_shape(),
-                   "Cannot reconstruct a Coefficient with a different value shape.")
-        return self._reconstruct(element, count)
-
-    def _reconstruct(self, element, count):
-        # This code is class specific
-        return Coefficient(element, count)
-
-    def element(self):
-        return self._element
-
     @property
     def ufl_shape(self):
-        return self._element.value_shape()
+        return self._ufl_shape
+
+    def ufl_function_space(self):
+        "Get the function space of this coefficient."
+        return self._ufl_function_space
+
+    def ufl_domain(self):
+        "Shortcut to get the domain of the function space of this coefficient."
+        return self._ufl_function_space.ufl_domain()
+
+    def ufl_element(self):
+        "Shortcut to get the finite element of the function space of this coefficient."
+        return self._ufl_function_space.ufl_element()
+
+    def element(self):
+        deprecate("Coefficient.element() is deprecated, please use Coefficient.ufl_element() instead.")
+        return self.ufl_element()
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
-        return self._element.is_cellwise_constant()
+        return self.ufl_element().is_cellwise_constant()
 
-    def domains(self):
+    def ufl_domains(self):
         "Return tuple of domains related to this terminal object."
-        return self._element.domains()
+        return self._ufl_function_space.ufl_domains()
 
-    def signature_data(self, renumbering):
+    def _ufl_signature_data_(self, renumbering):
         "Signature data for form arguments depend on the global numbering of the form arguments and domains."
         count = renumbering[self]
-        edata = self.element().signature_data(renumbering)
-        d = self.domain()
-        ddata = None if d is None else d.signature_data(renumbering)
-        return ("Coefficient", count, edata, ddata)
+        fsdata = self._ufl_function_space._ufl_signature_data_(renumbering)
+        return ("Coefficient", count, fsdata)
 
     def __str__(self):
         count = str(self._count)
@@ -102,8 +104,6 @@ class Coefficient(FormArgument):
             return "w_{%s}" % count
 
     def __repr__(self):
-        if self._repr is None:
-            self._repr = "Coefficient(%r, %r)" % (self._element, self._count)
         return self._repr
 
     def __eq__(self, other):
@@ -112,28 +112,34 @@ class Coefficient(FormArgument):
         if self is other:
             return True
         return (self._count == other._count and
-                self._element == other._element)
+                self._ufl_function_space == other._ufl_function_space)
 
 # --- Helper functions for defining constant coefficients without specifying element ---
 
 def Constant(domain, count=None):
     """UFL value: Represents a globally constant scalar valued coefficient."""
-    e = FiniteElement("Real", domain, 0)
-    return Coefficient(e, count=count)
+    domain = as_domain(domain)
+    element = FiniteElement("Real", domain.ufl_cell(), 0)
+    fs = FunctionSpace(domain, element)
+    return Coefficient(fs, count=count)
 
 def VectorConstant(domain, dim=None, count=None):
     """UFL value: Represents a globally constant vector valued coefficient."""
-    e = VectorElement("Real", domain, 0, dim)
-    return Coefficient(e, count=count)
+    domain = as_domain(domain)
+    element = VectorElement("Real", domain.ufl_cell(), 0, dim)
+    fs = FunctionSpace(domain, element)
+    return Coefficient(fs, count=count)
 
 def TensorConstant(domain, shape=None, symmetry=None, count=None):
     """UFL value: Represents a globally constant tensor valued coefficient."""
-    e = TensorElement("Real", domain, 0, shape=shape, symmetry=symmetry)
-    return Coefficient(e, count=count)
+    domain = as_domain(domain)
+    element = TensorElement("Real", domain.ufl_cell(), 0, shape=shape, symmetry=symmetry)
+    fs = FunctionSpace(domain, element)
+    return Coefficient(fs, count=count)
 
 # --- Helper functions for subfunctions on mixed elements ---
 
-def Coefficients(element):
+def Coefficients(function_space):
     """UFL value: Create a Coefficient in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
-    return split(Coefficient(element))
+    return split(Coefficient(function_space))
