@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 """Functions implementing compound expressions as equivalent representations using basic operators."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2015 Martin Sandve Alnæs and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -21,7 +22,7 @@
 
 from ufl.log import error, warning
 from ufl.assertions import ufl_assert
-from ufl.core.multiindex import indices
+from ufl.core.multiindex import indices, Index
 from ufl.tensors import as_tensor, as_matrix, as_vector
 from ufl.operators import sqrt
 
@@ -35,7 +36,6 @@ from ufl.operators import sqrt
 # would be a major improvement to UFL and the form compiler toolchain.
 # It could easily be a moderate to major undertaking to get rid of though.
 
-
 def cross_expr(a, b):
     assert len(a) == 3
     assert len(b) == 3
@@ -43,15 +43,29 @@ def cross_expr(a, b):
         return a[i]*b[j] - a[j]*b[i]
     return as_vector((c(1, 2), c(2, 0), c(0, 1)))
 
-
-def pseudo_determinant_expr(A):
+def generic_pseudo_determinant_expr(A):
     """Compute the pseudo-determinant of A: sqrt(det(A.T*A))."""
     i, j, k = indices(3)
     ATA = as_tensor(A[k, i]*A[k, j], (i, j))
     return sqrt(determinant_expr(ATA))
 
+def pseudo_determinant_expr(A):
+    """Compute the pseudo-determinant of A."""
+    m, n = A.ufl_shape
+    if n == 1:
+        # Special case 1xm for simpler expression
+        i = Index()
+        return sqrt(A[i,0]*A[i,0])
+    elif n == 2 and m == 3:
+        # Special case 2x3 for simpler expression
+        c = cross_expr(A[:,0], A[:,1])
+        i = Index()
+        return sqrt(c[i]*c[i])
+    else:
+        # Generic formulation based on A.T*A
+        return generic_pseudo_determinant_expr(A)
 
-def pseudo_inverse_expr(A):
+def generic_pseudo_inverse_expr(A):
     """Compute the Penrose-Moore pseudo-inverse of A: (A.T*A)^-1 * A.T."""
     i, j, k = indices(3)
     ATA = as_tensor(A[k, i]*A[k, j], (i, j))
@@ -59,9 +73,20 @@ def pseudo_inverse_expr(A):
     q, r, s = indices(3)
     return as_tensor(ATAinv[r, q] * A[s, q], (r, s))
 
+def pseudo_inverse_expr(A):
+    """Compute the Penrose-Moore pseudo-inverse of A: (A.T*A)^-1 * A.T."""
+    m, n = A.ufl_shape
+    if n == 1:
+        # Simpler special case for 1d
+        i, j, k = indices(3)
+        return as_tensor(A[i,j], (j,i)) / (A[k,0]*A[k,0])
+    else:
+        # Generic formulation
+        return generic_pseudo_inverse_expr(A)
+
 
 def determinant_expr(A):
-    "Compute the determinant of A."
+    "Compute the (pseudo-)determinant of A."
     sh = A.ufl_shape
     if sh == ():
         return A
@@ -84,11 +109,24 @@ def _det_2x2(B, i, j, k, l):
 def determinant_expr_2x2(B):
     return _det_2x2(B, 0, 1, 0, 1)
 
-def determinant_expr_3x3(A):
+def old_determinant_expr_3x3(A):
     return (A[0, 0]*_det_2x2(A, 1, 2, 1, 2) +
             A[0, 1]*_det_2x2(A, 1, 2, 2, 0) +
             A[0, 2]*_det_2x2(A, 1, 2, 0, 1))
 
+def determinant_expr_3x3(A):
+    return codeterminant_expr_nxn(A, [0,1,2], [0,1,2])
+
+def codeterminant_expr_nxn(A, rows, cols):
+    if len(rows) == 2:
+        return _det_2x2(A, rows[0], rows[1], cols[0], cols[1])
+    codet = 0.0
+    r = rows[0]
+    subrows = rows[1:]
+    for i, c in enumerate(cols):
+        subcols = cols[i+1:] + cols[:i]
+        codet += A[r, c] * codeterminant_expr_nxn(A, subrows, subcols)
+    return codet
 
 def inverse_expr(A):
     "Compute the inverse of A."
