@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 "The Equation class, used to express equations like a == L."
 
-# Copyright (C) 2012-2014 Anders Logg and Martin Sandve Alnes
+# Copyright (C) 2012-2015 Anders Logg and Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -17,7 +18,10 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 
-class Equation:
+# Export list for ufl.classes
+__all_classes__ = ["Equation"]
+
+class Equation(object):
     """This class is used to represent equations expressed by the "=="
     operator. Examples include a == L and F == 0 where a, L and F are
     Form objects."""
@@ -28,7 +32,14 @@ class Equation:
         self.rhs = rhs
 
     def __bool__(self):
-        "Evaluate bool(lhs_form == rhs_form)."
+        """Evaluate bool(lhs_form == rhs_form).
+
+        This will not trigger when setting 'equation = a == L',
+        but when e.g. running 'if equation:'.
+        """
+        # NB!: pep8 will say you should use isinstance here, but we do
+        #      actually want to compare the exact types in this case.
+        # Not equal if types are not identical (i.e. not accepting subclasses)
         if type(self.lhs) != type(self.rhs):
             return False
         # Try to delegate to equals function
