@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 """Compound tensor algebra operations."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2015 Martin Sandve Alnæs
 #
 # This file is part of UFL.
 #
@@ -104,7 +105,7 @@ class Transposed(CompoundTensorOperator):
 
     def __init__(self, A):
         CompoundTensorOperator.__init__(self, (A,))
-        ufl_assert(A.rank() == 2, "Transposed is only defined for rank 2 tensors.")
+        ufl_assert(len(A.ufl_shape) == 2, "Transposed is only defined for rank 2 tensors.")
 
     @property
     def ufl_shape(self):
@@ -196,8 +197,8 @@ class Dot(CompoundTensorOperator):
 
         # Checks
         ufl_assert((ar >= 1 and br >= 1) or scalar,
-            "Dot product requires non-scalar arguments, "\
-            "got arguments with ranks %d and %d." % \
+            "Dot product requires non-scalar arguments, "
+            "got arguments with ranks %d and %d." %
             (ar, br))
         ufl_assert(scalar or ash[-1] == bsh[0], "Dimension mismatch in dot product.")
 
@@ -266,7 +267,7 @@ class Trace(CompoundTensorOperator):
 
     def __new__(cls, A):
         # Checks
-        ufl_assert(A.rank() == 2, "Trace of tensor with rank != 2 is undefined.")
+        ufl_assert(len(A.ufl_shape) == 2, "Trace of tensor with rank != 2 is undefined.")
 
         # Simplification
         if isinstance(A, Zero):
